"""Private optional import utils.

Not for use outside trulens namespace.
"""

from trulens.core.utils.imports import format_import_errors

# Optional app types:
REQUIREMENT_INSTRUMENT_LLAMA = format_import_errors(
    "trulens-apps-llamaindex", purpose="instrumenting LlamaIndex apps"
)
REQUIREMENT_INSTRUMENT_LANGCHAIN = format_import_errors(
    "trulens-apps-langchain", purpose="instrumenting LangChain apps"
)
REQUIREMENT_INSTRUMENT_NEMO = format_import_errors(
    "trulens-apps-nemo", purpose="instrumenting NeMo Guardrails apps"
)

REQUIREMENT_SNOWFLAKE = format_import_errors(
    [
        "snowflake-core",
        "snowflake-connector-python",
        "snowflake-snowpark-python",
        "snowflake-sqlalchemy",
    ],
    purpose="connecting to Snowflake",
)



"""Serializable dataset-related classes."""

from __future__ import annotations

import logging
from typing import Hashable, Optional

from trulens.core.schema import types as mod_types_schema
from trulens.core.utils import serial
from trulens.core.utils.json import jsonify
from trulens.core.utils.json import obj_id_of_obj

logger = logging.getLogger(__name__)


class Dataset(serial.SerialModel, Hashable):
    """The class that holds the metadata of a dataset stored in the DB."""

    dataset_id: mod_types_schema.DatasetID  # str

    name: str

    meta: mod_types_schema.Metadata  # dict

    def __init__(
        self,
        name: str,
        dataset_id: Optional[mod_types_schema.DatasetID] = None,
        meta: Optional[mod_types_schema.Metadata] = None,
        **kwargs,
    ):
        kwargs["name"] = name
        kwargs["meta"] = meta if meta is not None else {}
        super().__init__(
            dataset_id="temporary", **kwargs
        )  # dataset_id will be updated below

        if dataset_id is None:
            dataset_id = obj_id_of_obj(jsonify(self), prefix="dataset")

        self.dataset_id = dataset_id

    def __hash__(self):
        return hash(self.dataset_id)


# HACK013: Need these if using __future__.annotations .
Dataset.model_rebuild()



"""# Trulens Core LLM Evaluation Library

The `trulens-core` library includes everything to get started.

"""

from importlib.metadata import version
import os

from trulens.core.feedback import Feedback
from trulens.core.feedback import Provider
from trulens.core.feedback import SnowflakeFeedback
from trulens.core.schema import FeedbackMode
from trulens.core.schema import Select
from trulens.core.session import Tru
from trulens.core.session import TruSession
from trulens.core.utils.imports import safe_importlib_package_name

# NOTE: workaround for MKL and multiprocessing
# https://github.com/pytorch/csprng/issues/115
os.environ["KMP_DUPLICATE_LIB_OK"] = "TRUE"

__version__ = version(safe_importlib_package_name(__package__ or __name__))


__all__ = [
    "Tru",  # aliases TruSession for backwards compatibility
    "TruSession",  # main interface
    # app setup
    "FeedbackMode",
    # feedback setup
    "Feedback",
    "SnowflakeFeedback",
    "Select",
    "Provider",
]
